#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import time
import argparse

import technoap.device.apg7400a as usbmca


##############################################################
def main(args):
	
	print ("starting!!!")

	try:
		#open device
		serialnum = ("APG7400AA").encode('utf-8')
		handle = usbmca.apg7400a.APG7400A.OpenDeviceBySerialNumber(serialnum)

		if handle is not None:
			#command the device to initialize
			handle.InitialDevice()

			#configuration
			config = handle.createConfigObject()
			ret1st = init_config(config)	#writing parameter

			if ret1st == True:	#configuration succeeded
				acquisition(handle, config, 5.0, args.file)		###
			else:
				print ("configuration error...")
		else:
			print ("device open error(%s)..." % (serialnum))
	except Exception as ex:
		print (ex)

	print ("complete...")


##############################################################
def acquisition(handle, config, acq_sec, savefile):

	dummy = config.stopAcquisition()		#Issue a stop command once considering that the measurement is continuing
	dummy = config.clearAcquisitionData()	#Clear the data held on the device side

	dummy = config.setAcquisitionMode(usbmca.constant.PARAM_MOD_HIST)			#histogram mode
	dummy = config.setAcquisitionTimeMode(usbmca.constant.PARAM_MMD_REALTIME)	#real time mode
	dummy = config.setAcquisitionTime(acq_sec)	#setting acquisition time(seconds)

	elapsedTime = -0.001	#initialize elapsed time
	status = handle.createStatusObject()		#status object

	#start!!!
	dummy = config.startAcquisition()

	########## acquisition ##########
	while acq_sec >= elapsedTime:
		status.readStatus()		#reading status
		elapsedTime = status.getElapsedSeconds()

		print ("\n##### %.1f/%.1f(sec) #####" % (elapsedTime, acq_sec))
		print ("live time(nano sec):    %d, %d, %d, %d" % (
			status.getLiveTimeNanoseconds(usbmca.constant.CHNUM_1ST),	#CH1
			status.getLiveTimeNanoseconds(usbmca.constant.CHNUM_2ND),	#CH2
			status.getLiveTimeNanoseconds(usbmca.constant.CHNUM_3RD),	#CH3
			status.getLiveTimeNanoseconds(usbmca.constant.CHNUM_4TH)))	#CH4
		print ("dead time(nano sec):    %d, %d, %d, %d" % (
			status.getDeadTimeNanoseconds(usbmca.constant.CHNUM_1ST),	#CH1
			status.getDeadTimeNanoseconds(usbmca.constant.CHNUM_2ND),	#CH2
			status.getDeadTimeNanoseconds(usbmca.constant.CHNUM_3RD),	#CH3
			status.getDeadTimeNanoseconds(usbmca.constant.CHNUM_4TH)))	#CH4
		print ("throughput count rate:  %d, %d, %d, %d" % (
			status.mStatus["TCR1",], status.mStatus["TCR2",],	#CH1, CH2
			status.mStatus["TCR3",], status.mStatus["TCR4",]))	#CH3, CH4
		print ("throughput total count: %d, %d, %d, %d" % (
			status.mStatus["TCT1",], status.mStatus["TCT2",],	#CH1, CH2
			status.mStatus["TCT3",], status.mStatus["TCT4",]))	#CH3, CH4
		print ("input count rate:       %d, %d, %d, %d" % (
			status.mStatus["ICR1",], status.mStatus["ICR2",],	#CH1, CH2
			status.mStatus["ICR3",], status.mStatus["ICR4",]))	#CH3, CH4
		time.sleep(0.5)		#delay
	
	#Issue a acquisition stop command upon completion
	dummy = config.stopAcquisition()

	#################################################
	########## read histogram --> CSV file ##########
	#################################################
	histdata = []
	histogram = handle.createHistogramObject()	#histogram object

	for i in range(usbmca.constant.CHNUM_MAX):
		ret0th, hist0 = histogram.read(i + 1)	#reading histogram data
		histdata.append(hist0)

	with open(savefile, 'w') as csv:
		csv.writelines("[header]\n")
		csv.writelines("real time(s),%f\n" % (elapsedTime))

		csv.writelines("[data]\n")
		csv.writelines("#bin,CH1,CH2,CH3,CH4\n")

		for i in range(8192):
			textline = "%d,%d,%d,%d,%d\n" % (i,
				histdata[0][i], histdata[1][i],		#CH1, CH2
				histdata[2][i], histdata[3][i])		#CH3, CH4
			csv.writelines(textline)

	return

##############################################################
def init_config(config):

	###### common #####
	config.mParameter["PDSW",] = usbmca.constant.PARAM_PDS_ABS	#peek detect --> absolution

	###### CH #####
	#ADC gain
	config.mParameter["ADG",] = [
		usbmca.constant.PARAM_ADG_04K,	#CH1 --> 4096bin
		usbmca.constant.PARAM_ADG_04K,	#CH2 --> 4096bin
		usbmca.constant.PARAM_ADG_04K,	#CH3 --> 4096bin
		usbmca.constant.PARAM_ADG_04K]	#CH4 --> 4096bin
	#Set the following parameters in the order of [CH1, CH2, CH3, CH4]
	config.mParameter["THR",] = [80, 80, 80, 80]			#threshold(ch)
	config.mParameter["LLD",] = [95, 95, 95, 95]			#lower level discrimination(ch)
	config.mParameter["ULD",] = [8191, 8191, 8191, 8191]	#upper level discrimination(ch)
	config.mParameter["OFS",] = [0, 0, 0, 0]				#offset(ch)

	### writng parameter ###
	result = config.writeParameterAll()

	return result

##############################################################
if __name__ == '__main__':
	msg_file = 'Specify the file (CSV format) to save the spectrum data. If not specified, use ./usbmca.csv file.'
	parser = argparse.ArgumentParser(description='Acquisition in histogram mode.')
	parser.add_argument('-f', '--file', default='./usbmca.csv', help=msg_file)
	args = parser.parse_args()

	ret1st = usbmca.apg7400a.startup()		#Be sure to call when the application starts

	if ret1st == True:
		main(args)		#main processing
		ret2nd = usbmca.apg7400a.cleanup()	#Be sure to call when the application ends
