#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import time
import argparse

import technoap.device.apg7400a as usbmca
from technoap.time.stopwatch import Stopwatch


##############################################################
def main(args):
	
	print ("starting!!!")

	try:
		#open device
		serialnum = ("APG7400AA").encode('utf-8')
		handle = usbmca.apg7400a.APG7400A.OpenDeviceBySerialNumber(serialnum)

		if handle is not None:
			#command the device to initialize
			handle.InitialDevice()

			#configuration
			config = handle.createConfigObject()
			ret1st = init_config(config)	#writing parameter

			if ret1st == True:	#configuration succeeded
				acquisition(handle, config, 5.0, args.file)		###
			else:
				print ("configuration error...")
		else:
			print ("device open error(%s)..." % (serialnum))
	except Exception as ex:
		print (ex)

	print ("complete...")


##############################################################
def acquisition(handle, config, acq_sec, savefile):

	dummy = config.stopAcquisition()		#Issue a stop command once considering that the measurement is continuing
	dummy = config.clearAcquisitionData()	#Clear the data held on the device side

	dummy = config.setAcquisitionMode(usbmca.constant.PARAM_MOD_LIST)			#list mode
	dummy = config.setAcquisitionTimeMode(usbmca.constant.PARAM_MMD_REALTIME)	#real time mode
	dummy = config.setAcquisitionTime(acq_sec)	#setting acquisition time(seconds)

	# Open the save destination file
	with open(savefile, 'wb') as binfile:
		timerDisplay = Stopwatch()		#create a timer object to display the status
		timerDisplay.start()

		elapsedTime = -0.001	#initialize elapsed time
		status = handle.createStatusObject()		#status object
		listData = handle.createListdataObject()	#listdata object

		#start!!!
		dummy = config.startAcquisition()

		########## acquisition ##########
		while acq_sec >= elapsedTime:
			status.readStatus()		#reading status
			elapsedTime = status.getElapsedSeconds()

			#Display the status under the following conditions
			#  (a) 1 second cycle
			#  (b) The elapsed time has expired
			if timerDisplay.getElapsedSecond() >= 1.0 or elapsedTime >= acq_sec:
				print ("\n##### %.1f/%.1f(sec) #####" % (elapsedTime, acq_sec))
				print ("live time(nano sec):    %d, %d, %d, %d" % (
					status.getLiveTimeNanoseconds(usbmca.constant.CHNUM_1ST),	#CH1
					status.getLiveTimeNanoseconds(usbmca.constant.CHNUM_2ND),	#CH2
					status.getLiveTimeNanoseconds(usbmca.constant.CHNUM_3RD),	#CH3
					status.getLiveTimeNanoseconds(usbmca.constant.CHNUM_4TH)))	#CH4
				print ("dead time(nano sec):    %d, %d, %d, %d" % (
					status.getDeadTimeNanoseconds(usbmca.constant.CHNUM_1ST),	#CH1
					status.getDeadTimeNanoseconds(usbmca.constant.CHNUM_2ND),	#CH2
					status.getDeadTimeNanoseconds(usbmca.constant.CHNUM_3RD),	#CH3
					status.getDeadTimeNanoseconds(usbmca.constant.CHNUM_4TH)))	#CH4
				print ("throughput count rate:  %d, %d, %d, %d" % (
					status.mStatus["TCR1",], status.mStatus["TCR2",],	#CH1, CH2
					status.mStatus["TCR3",], status.mStatus["TCR4",]))	#CH3, CH4
				print ("throughput total count: %d, %d, %d, %d" % (
					status.mStatus["TCT1",], status.mStatus["TCT2",],	#CH1, CH2
					status.mStatus["TCT3",], status.mStatus["TCT4",]))	#CH3, CH4
				print ("input count rate:       %d, %d, %d, %d" % (
					status.mStatus["ICR1",], status.mStatus["ICR2",],	#CH1, CH2
					status.mStatus["ICR3",], status.mStatus["ICR4",]))	#CH3, CH4
				timerDisplay.clear()	#Clear elapsed time

			rxbytes = read_listdata(listData, binfile)	#read and save listdata
			time.sleep(0.01)		#delay

		#Retrieve all remaining data
		rxbytes = 1
		while rxbytes > 0:
			rxbytes = read_listdata(listData, binfile)

	#Issue a acquisition stop command upon completion
	dummy = config.stopAcquisition()

	return

##############################################################
def read_listdata(listData, binfile):

	rxdata, rxbytes = listData.read()

	if rxbytes > 0:
		binfile.write(rxdata)

	return rxbytes

##############################################################
def init_config(config):

	###### common #####
	config.mParameter["PDSW",] = usbmca.constant.PARAM_PDS_ABS	#peek detect --> absolution

	###### CH #####
	#ADC gain
	config.mParameter["ADG",] = [
		usbmca.constant.PARAM_ADG_04K,	#CH1 --> 4096bin
		usbmca.constant.PARAM_ADG_04K,	#CH2 --> 4096bin
		usbmca.constant.PARAM_ADG_04K,	#CH3 --> 4096bin
		usbmca.constant.PARAM_ADG_04K]	#CH4 --> 4096bin
	#Set the following parameters in the order of [CH1, CH2, CH3, CH4]
	config.mParameter["THR",] = [80, 80, 80, 80]			#threshold(ch)
	config.mParameter["LLD",] = [95, 95, 95, 95]			#lower level discrimination(ch)
	config.mParameter["ULD",] = [8191, 8191, 8191, 8191]	#upper level discrimination(ch)
	config.mParameter["OFS",] = [0, 0, 0, 0]				#offset(ch)

	### writng parameter ###
	result = config.writeParameterAll()

	return result

##############################################################
if __name__ == '__main__':
	msg_file = 'Specify the file (BIN format) to save the event data. If not specified, use ./usbmca.bin file.'
	parser = argparse.ArgumentParser(description='Acquisition in list mode.')
	parser.add_argument('-f', '--file', default='./usbmca.bin', help=msg_file)
	args = parser.parse_args()

	ret1st = usbmca.apg7400a.startup()		#Be sure to call when the application starts

	if ret1st == True:
		main(args)		#main processing
		ret2nd = usbmca.apg7400a.cleanup()	#Be sure to call when the application ends
