#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import ctypes
from ctypes import *
from ..device import device

from . import struct
from . import constant


class FTDIBase(object):

	LIBHANDLE = constant.FTD2XXLIBHANDLE

	#####################################################
	@staticmethod
	def isSuccess(value):
		return constant.FT_SUCCESS(value)

	#####################################################
	@staticmethod
	def SetVIDPID(vendorID, productID):

		result = constant.FT_OK

		if device.getOperatingSystem() == device.OS_TYPE_LINUX:
			value1 = ctypes.c_uint(vendorID)
			value2 = ctypes.c_uint(productID)
			result = FTDIBase.LIBHANDLE.FT_SetVIDPID(value1, value2)

		return result

	#####################################################
	@staticmethod
	def GetVIDPID():

		result = constant.FT_OK
		vid = ctypes.c_uint(0)
		pid = ctypes.c_uint(0)

		if device.getOperatingSystem() == device.OS_TYPE_LINUX:
			result = FTDIBase.LIBHANDLE.FT_GetVIDPID(byref(vid), byref(pid))

		vendorID = int(vid.value)
		productID = int(pid.value)

		return result, vendorID, productID

	#####################################################
	def __init__(self):
		self.mHandle = None
		self.__initializing()

	#####################################################
	def __del__(self):
		self.Close()

	#####################################################
	def __initializing(self):
		self.Close()
		self.mHandle = None
		self.mDeviceCounts = 0
		self.mDeviceList = []

	#####################################################
	def OpenBySerialNumber(self, serialnum):
		handle = ctypes.c_void_p()
		param = ctypes.c_uint(constant.FT_OPEN_BY_SERIAL_NUMBER)
		result = FTDIBase.LIBHANDLE.FT_OpenEx(serialnum, param, byref(handle))

		if constant.FT_SUCCESS(result) == True:
			self.__initializing()
			self.mHandle = handle

		return result

	#####################################################
	def OpenByIndex(self, index):
		handle = ctypes.c_void_p()
		result = FTDIBase.LIBHANDLE.FT_Open(index, byref(handle))
		
		if constant.FT_SUCCESS(result) == True:
			self.__initializing()
			self.mHandle = handle

		return result

	#####################################################
	def Close(self):
		if self.mHandle is not None:
			result = FTDIBase.LIBHANDLE.FT_Close(self.mHandle)
			self.mHandle = None
		else:
			result = constant.FT_OTHER_ERROR
		return result
	
	#####################################################
	@staticmethod
	def GetDeviceInfoList0():

		deviceCounts = 0
		deviceList = []

		counts = ctypes.c_uint()
		result = FTDIBase.LIBHANDLE.FT_CreateDeviceInfoList(byref(counts))

		if result == constant.FT_OK and counts.value > 0:
			template0 = struct.FT_DEVICE_LIST_INFO_NODE * counts.value
			info = template0()
			result0 = FTDIBase.LIBHANDLE.FT_GetDeviceInfoList(info, byref(counts))

			if result0 == constant.FT_OK:
				deviceCounts = int(counts.value)
				for i in range(deviceCounts):
					deviceList.append(info[i])

		return result, deviceList, deviceCounts

	#####################################################
	def GetDeviceInfoList(self):

		result, self.mDeviceList, self.mDeviceCounts = self.GetDeviceInfoList0()
		return result, self.mDeviceList, self.mDeviceCounts

	#####################################################
	@staticmethod
	def GetDeviceInfoDetail(index):

		info = None

		flags = ctypes.c_uint(0)
		type0 = ctypes.c_uint(0)
		id = ctypes.c_uint(0)
		locid = ctypes.c_uint(0)
		template0 = c_byte * 16
		serialnum = template0()
		template1 = c_byte * 64
		description = template1()
		handle = ctypes.c_void_p()
		result = FTDIBase.LIBHANDLE.FT_GetDeviceInfoDetail(index, byref(flags), byref(type0), byref(id), byref(locid), serialnum, description, byref(handle))

		if constant.FT_SUCCESS(result) == True:
			info = struct.FT_DEVICE_DETAIL_INFO_NODE()
			info.mFlags = flags.value
			info.mType = type0.value
			info.mID = id.value
			info.mLocId = locid.value
			info.mSerialNumber = struct.cbytearraytotext(serialnum)
			info.mDescription = struct.cbytearraytotext(description)
			info.mHandle = handle
		
		return result, info

	#####################################################
	def Read(self, rxbytes):
		
		rxdata = None
		
		template0 = ctypes.c_ubyte * rxbytes
		buffer = template0()
		bufferbytes = ctypes.c_uint(rxbytes)
		readBytes = ctypes.c_uint(0)
		result = FTDIBase.LIBHANDLE.FT_Read(self.mHandle, buffer, bufferbytes, byref(readBytes))

		if constant.FT_SUCCESS(result) == True:
			rxdata = bytearray(readBytes.value)
			for i in range(readBytes.value):
				rxdata[i] = buffer[i]

		return result, rxdata, int(readBytes.value)

	#####################################################
	def Write(self, buffer, txbytes):

		templete0 = ctypes.c_ubyte * txbytes
		txdata = templete0()

		for i in range(txbytes):
			txdata[i] = buffer[i]

		bufferbytes = ctypes.c_uint(txbytes)
		writtenBytes = ctypes.c_byte(0)
		result = FTDIBase.LIBHANDLE.FT_Write(self.mHandle, txdata, bufferbytes, byref(writtenBytes))
		return result, int(writtenBytes.value)


	#####################################################
	def ResetDevice(self):
		return FTDIBase.LIBHANDLE.FT_ResetDevice(self.mHandle)

	#####################################################
	def SetBitMode(self, mask, enable):
		value1 = ctypes.c_byte(mask)
		value2 = ctypes.c_byte(enable)
		return FTDIBase.LIBHANDLE.FT_SetBitMode(self.mHandle, value1, value2)

	#####################################################
	def SetLatencyTimer(self, latency):
		value = ctypes.c_byte(latency)
		return FTDIBase.LIBHANDLE.FT_SetLatencyTimer(self.mHandle, value)

	#####################################################
	def SetTimeouts(self, readTimeouts, writeTimeouts):
		value1 = ctypes.c_uint(readTimeouts)
		value2 = ctypes.c_uint(writeTimeouts)
		return FTDIBase.LIBHANDLE.FT_SetTimeouts(self.mHandle, value1, value2)

	#####################################################
	def SetFlowControl(self, flowControl, xOn, xOff):
		value1 = ctypes.c_ushort(flowControl)
		value2 = ctypes.c_byte(xOn)
		value3 = ctypes.c_byte(xOff)
		return FTDIBase.LIBHANDLE.FT_SetFlowControl(self.mHandle, value1, value2, value3)

	#####################################################
	def Purge(self, mask):
		value = ctypes.c_uint(mask)
		return FTDIBase.LIBHANDLE.FT_Purge(self.mHandle, value)
